/*********************************************************************
 *
 *                  A50 akord app
 *                  A50akord
 *
 * v.1.0 - 08.08.2021, Midi I/O, L.Pekny
 ********************************************************************/

#include <avr/io.h>
#include "Pin.h"
#include "MTimer.h"
#include "UartBuf.h"
#include "Midi.h"
#include "A50akord.h"


const BYTE GMlist[12] =
{
	SquarLead,Clarinet,Flute,TenorSax,
	String,Violin,Piano,
	DrawOrgan,RockOrgan,Accordion,Harmonica,
	GMnone
};


BYTE VoiceEnable, VoicePreset;								// Preset
BYTE KeyInstrument, Volume;									// Keyboard Instrument, Volume
BYTE Reverb, Tremolo, Chorus;								// Keyboard effects
BYTE PreInstrument, Instrument;											// Instrument Preset
BYTE flPreset;


void AKORDopen(BYTE Ux)
{
	MIDIopen(Ux);											// Open Com, 31250 Bd
	VoiceEnable=-1;	VoicePreset=prLMMH;
	PreInstrument=GMnone-1; Volume=100;
	Reverb=12; Chorus=0;
	KeyInstrument=Piano-1;
	flPreset=0;
}


void AKORDclose(BYTE Ux)
{
	MIDIclose(Ux);
}


void AKORDread(BYTE Ux)
{
	MIDImsg=MIDIread(Ux);
}


void AKORDwrite(BYTE Ux, BYTE Cmd, BYTE Dat1L, BYTE Dat2H)
{
	BYTE Size;

	if (Cmd<NoteOff) {return;}
	UARTwrite(Ux,Cmd);
	Size=MIDIsize(Cmd);
	if (Size) {UARTwrite(Ux,Dat1L);}
	if (Size==2) {UARTwrite(Ux,Dat2H);}
}


void AKORDsetChanel(BYTE Ux, BYTE Preset)					// Set Channel Volume, Pitch, Atack
{
	switch(Preset & 0x0F)									// Convert Preset to VoiceEnable
	{
	  case prLM: VoiceEnable=VoiceL+VoiceMP;
	  	break;
	  case prMM: VoiceEnable=VoiceM+VoiceMP;
	  	break;
	  case prHM: VoiceEnable=VoiceH+VoiceMP;
	  	break;
	  case prLMH: VoiceEnable=VoiceL+VoiceMP+VoiceH;
	  	break;
	  case prLMM: VoiceEnable=VoiceL+VoiceMP+VoiceM;
	  	break;
	  case prHMM: VoiceEnable=VoiceH+VoiceMP+VoiceM;
	  	break;
	  case prLMMH: VoiceEnable=VoiceL+VoiceMP+VoiceM+VoiceH;
	  	break;
	  case prMMM: VoiceEnable=VoiceM+VoiceMP+VoiceMB;
	  	break;
	  case prL: VoiceEnable=VoiceL;
	  	break;
	  case prM: VoiceEnable=VoiceM;
	  	break;
	  case prH: VoiceEnable=VoiceH;
	  	break;
	  case prOff: VoiceEnable=0;
	  	break;
	  default: ;
	  	break;
	}

	if (PreInstrument==GMnone-1) {Instrument=KeyInstrument;}
	else {Instrument=PreInstrument;}

	if (VoiceEnable & VoiceM)								// Voice Mid
	{														// Set Instrument, Volume
	  AKORDwrite(Ux,ControlChange+ChanM,BankSelect,0);		// Bank 0
	  AKORDwrite(Ux,ControlChange+ChanM,BankSelect+32,0);
	  AKORDwrite(Ux,ProgramChange+ChanM,Instrument,0);		// Instrument-1
	  AKORDwrite(Ux,ControlChange+ChanM,ChannelVolume,Volume);
	  AKORDwrite(Ux,ControlChange+ChanM,Effects1,Reverb);
	  AKORDwrite(Ux,ControlChange+ChanM,Effects3,Chorus);
	}

	if (VoiceEnable & VoiceMP)								// Voice Mid+Pitch
	{														// Set Instrument, Volume-1/8, Pitch A4+4Hz 
	  AKORDwrite(Ux,ControlChange+ChanMP,RPNLSB,1);
	  AKORDwrite(Ux,ControlChange+ChanMP,RPNMSB,0);
	  AKORDwrite(Ux,ControlChange+ChanMP,DataEntry,0x40+Detune);	// Fine Tune (+16c, range 100 cent/8192)

	  AKORDwrite(Ux,ControlChange+ChanMP,BankSelect,0);		// Bank 0
	  AKORDwrite(Ux,ControlChange+ChanMP,BankSelect+32,0);
	  AKORDwrite(Ux,ProgramChange+ChanMP,Instrument,0);
	  AKORDwrite(Ux,ControlChange+ChanMP,ChannelVolume,Volume-(Volume>>3));
	  AKORDwrite(Ux,ControlChange+ChanMP,Effects1,Reverb);
	  AKORDwrite(Ux,ControlChange+ChanMP,Effects3,Chorus);
	}

	if (VoiceEnable & VoiceMB)								// Voice Mid-Pitch
	{														// Set Instrument, Volume-1/8, Pitch A4-4Hz
	  AKORDwrite(Ux,ControlChange+ChanMB,RPNLSB,1);
	  AKORDwrite(Ux,ControlChange+ChanMB,RPNMSB,0);
	  AKORDwrite(Ux,ControlChange+ChanMB,DataEntry,0x40-Detune);	// Fine Tune (-16c range 100 cent/8192)

	  AKORDwrite(Ux,ControlChange+ChanMB,BankSelect,0);		// Bank 0
	  AKORDwrite(Ux,ControlChange+ChanMB,BankSelect+32,0);
	  AKORDwrite(Ux,ProgramChange+ChanMB,Instrument,0);
	  AKORDwrite(Ux,ControlChange+ChanMB,ChannelVolume,Volume-(Volume>>3));
	  AKORDwrite(Ux,ControlChange+ChanMB,Effects1,Reverb);
	  AKORDwrite(Ux,ControlChange+ChanMB,Effects3,Chorus);
	}

	if (VoiceEnable & VoiceL)								// Voice Low
	{														// Set Instrument, Volume-1/4
	  AKORDwrite(Ux,ControlChange+ChanL,BankSelect,0);		// Bank 0
	  AKORDwrite(Ux,ControlChange+ChanL,BankSelect+32,0);
	  AKORDwrite(Ux,ProgramChange+ChanL,Instrument,0);
	  AKORDwrite(Ux,ControlChange+ChanL,ChannelVolume,Volume-(Volume>>2));
	  AKORDwrite(Ux,ControlChange+ChanL,Effects1,Reverb);
	  AKORDwrite(Ux,ControlChange+ChanL,Effects3,Chorus);
	}

	if (VoiceEnable & VoiceH)								// Voice Hi
	{														// Set Instrument, Volume-1/4
	  AKORDwrite(Ux,ControlChange+ChanH,BankSelect,0);		// Bank 0
	  AKORDwrite(Ux,ControlChange+ChanH,BankSelect+32,0);
	  AKORDwrite(Ux,ProgramChange+ChanH,Instrument,0);
	  AKORDwrite(Ux,ControlChange+ChanH,ChannelVolume,Volume-(Volume>>2));
	  AKORDwrite(Ux,ControlChange+ChanH,Effects1,Reverb);
	  AKORDwrite(Ux,ControlChange+ChanH,Effects3,Chorus);
	}
}


void AKORDonNoteOn(BYTE Ux, tMIDImsg Emsg)  				// Press Key, Keyboard channel
{
	tMIDImsg Vmsg;

	if ((Emsg.Cmd & 0x0F)!=ChanKey) {return;}				// Only default channel, keyboard

	if (flPreset)
	{
	  if (!isVTim(VTPROG,TimeOutProg))						// No Timeout?
	  {
	    if ((Emsg.Dat1L>=KeyC4)&&(Emsg.Dat1L<KeyC5))		// Range C4-H4 for Preset?
	    {
	      VoicePreset=Emsg.Dat1L-KeyC4+1;					// Get VoicePreset+1
	      AKORDsetChanel(Ux,VoicePreset);					// Set Channel
	    }
	  
	    if ((Emsg.Dat1L>=KeyC3)&&(Emsg.Dat1L<KeyC4))		// Range C3-H3 for Instrument?
	    {
	      PreInstrument=GMlist[Emsg.Dat1L-KeyC3]-1;			// Get Instrument-1
	      AKORDsetChanel(Ux,VoicePreset);					// Set Channel
	    }

		if (Emsg.Dat1L>=KeyC5) {flPreset=0;}				// End of Preset
	    return;												// No Timeout, continue Preset
	  } else {flPreset=0;}									// Timeout, End of Preset
	}

	SetLED;
	Emsg.Cmd&=0xF0;											// Without Ch
	Vmsg.Dat1L=Emsg.Dat1L;									// Key
	Vmsg.Dat2H=Emsg.Dat2H;									// Velocity

	if (VoiceEnable & VoiceM)
	{
	  Vmsg.Cmd=Emsg.Cmd+ChanM;								// Note On, Ch
 	  MIDIwrite(Ux,Vmsg);									// Voice M
	}

	if (VoiceEnable & VoiceMP)
	{
	  Vmsg.Cmd=Emsg.Cmd+ChanMP;								// Note On, ChMP
	  MIDIwrite(Ux,Vmsg);									// Voice MP
	}

	if (VoiceEnable & VoiceMB)
	{
	  Vmsg.Cmd=Emsg.Cmd+ChanMB;								// Note On, ChMB
	  MIDIwrite(Ux,Vmsg);									// Voice MP
	}

	if (VoiceEnable & VoiceL)
	{
	  if (Emsg.Dat1L>=12)
	  {
	    Vmsg.Cmd=Emsg.Cmd+ChanL;							// Note On, ChL
	    Vmsg.Dat1L=Emsg.Dat1L-12;							// Key octave low
	    MIDIwrite(Ux,Vmsg);									// Voice L
	  }
	}

	if (VoiceEnable & VoiceH)
	{
	  if (Emsg.Dat1L<=115)
	  {
	    Vmsg.Cmd=Emsg.Cmd+ChanH;							// Note On, CH
	    Vmsg.Dat1L=Emsg.Dat1L+12;							// Key octave high
	    MIDIwrite(Ux,Vmsg);									// Voice H
	  }
	}
	ClrLED;
}


void AKORDonNoteOff(BYTE Ux, tMIDImsg Emsg)  				// Press Key, Keyboard channel
{
	tMIDImsg Vmsg;

	if ((Emsg.Cmd & 0x0F)!=ChanKey) {return;}				// Only default channel, keyboard

	Emsg.Cmd&=0xF0;											// Without Ch
	Vmsg.Dat1L=Emsg.Dat1L;									// Key
	Vmsg.Dat2H=Emsg.Dat2H;									// Velocity

	Vmsg.Cmd=Emsg.Cmd+ChanM;								// Note Off, Ch
	MIDIwrite(Ux,Vmsg);										// Voice M

	Vmsg.Cmd=Emsg.Cmd+ChanMP;								// Note Off, ChMP
	MIDIwrite(Ux,Vmsg);										// Voice MP

	Vmsg.Cmd=Emsg.Cmd+ChanMB;								// Note Off, ChMB
	MIDIwrite(Ux,Vmsg);										// Voice MB

	if (Emsg.Dat1L>=12)
	{
	  Vmsg.Cmd=Emsg.Cmd+ChanL;								// Note Off, ChL
	  Vmsg.Dat1L=Emsg.Dat1L-12;								// Key octave low
 	  MIDIwrite(Ux,Vmsg);									// Voice L
	}

	if (Emsg.Dat1L<=115)
	{
	  Vmsg.Cmd=Emsg.Cmd+ChanH;								// Note Off, ChH
	  Vmsg.Dat1L=Emsg.Dat1L+12;								// Key octave high
 	  MIDIwrite(Ux,Vmsg);									// Voice H
	}
}


void AKORDonControl(BYTE Ux, tMIDImsg Emsg)					// Change Control
{
	StartVTim(VTPROG); flPreset=1;							// Start Timer for Preset

	if ((Emsg.Cmd & 0x0F)==ChanKey)							// Channel Keyboard?
	{
	  switch(Emsg.Dat1L)									// Get Controller data
	  {
	    case ChannelVolume: Volume=Emsg.Dat2H;
	    	break;
	    case Effects1: Reverb=Emsg.Dat2H;
	    	break;
	    case Effects2: Tremolo=Emsg.Dat2H;
	    	break;
	    case Effects3: Chorus=Emsg.Dat2H;
	  	  break;
	    default: ;
	  	  break;
	  }
	}
}


void AKORDonProgram(BYTE Ux, tMIDImsg Emsg)					// Change Program
{
	if ((Emsg.Cmd & 0x0F)!=ChanKey) {return;}				// Only default channel, keyboard

	KeyInstrument=Emsg.Dat1L;
}


void AKORDevent(BYTE Ux, tMIDImsg Emsg)
{
	switch(Emsg.Cmd & 0xF0)
	{
	  case NoteOn: AKORDonNoteOn(Ux,Emsg);
	  	break;
	  case NoteOff: AKORDonNoteOff(Ux,Emsg);
	  	break;
	  case ControlChange: AKORDonControl(Ux,Emsg);
	  	break;
	  case ProgramChange: AKORDonProgram(Ux,Emsg);
	  	break;
	  default: ;
	  	break;
	}
}


void AKORDtask(BYTE Ux)
{
	MIDImsg=MIDIread(Ux);
	AKORDevent(Ux,MIDImsg);
	MIDImsg.Cmd=InvalidMsg;
}
