/*********************************************************************
 *
 *                  MTimer
 *                  A50akord
 *
 * v.1.1 - 15.07.2021, L.Pekny - ATmega
 * v.1.0 - 27.08.2018, Timer, Delay, Virtual, L.Pekny
 ********************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
//#include <util/delay.h>
#include "MTimer.h"


DWORD Time1ms=0;
DWORD VTim[8]; 	 					// Virtual Timer 0..7
BYTE Prescal10=0;


void TIM1open(void)
{
	Time1ms=0;
	Prescal10= 0;

	OCR1A=cTim1per;					// Period value 1ms
	TCNT1=0;						// Clear Counter
    TCCR1A=cTim1cfgA;				// CTC, no output pin
	TCCR1B=cTim1cfgB;				// CTC, clk=Fcpu/1
 	TIMSK1=(1<<OCIE1A);				// Enable Timer1 interrupt
}


void TIM1close(void)
{
    TCCR1A=0;
    TCCR1B=0;      					// Disable timer1
    TIMSK1=0; 						// Disable the Timer1 interrupt
}


WORD TIM1read(void)
{
	return TCNT1;					// Read counter TCNTL1, TCNTH1, (0..999us)
}


void TIM1write(WORD T1period)
{
	OCR1A=T1period;
}


ISR(TIMER1_INT)						// Threads, 1 ms
{
	Time1ms++;   					// Inc clock [ms]

/*    if (Prescal10>0) {Prescal10--;}	// Prescaler 1:10
	else
	{
	  Prescal10= 10;				// 10 ms
	}
*/
}


void Delay_us(WORD us)				// max.1000 us, OCR1A reset period 1ms
{
    WORD targetCount;  
    WORD backupCount; 

    backupCount = TCNT1;  			// Backup current count
    targetCount = /*countPerMicroSec* */us-1;

	while ((TCNT1-backupCount) < targetCount);
}


void Delay_ms(WORD ms)
{
	WORD i;

	for (i=0; i<ms; i++) {Delay_us(1000);}
}


void StartVTim(BYTE index)
{
	VTim[index] = Time1ms;
}


char isVTim(BYTE index, DWORD waitms)
{
	return ((Time1ms-VTim[index]) > waitms);
}
