/*********************************************************************
 *
  *                  A50akord
 * Yamaha PS A50 - accordion, organ tone
 * ATmega328P, 8MHz
 *
 * BODLEVEL=5 (2.7V), CKSEL=15 (low Xt), SUT=1 (BOD+Xt), CLKPS=0 (Xt/1)
 * CLKDIV8=1, CKOUT=1, RSTDISBL=1, DWEN=1, WDTON=1, SPIEN=0 (IPS), EES=0
 * BOOTSZ=11 (256), BOOTRST=1 (0000).  F5,D7,DF
 *
 * v.1.0 - 30.07.2021, L.Pekny
 *********************************************************************/

#define F_CPU 8000000UL							// For dely.h frequency 8MHz
#include <avr/io.h>
#include <avr/fuse.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
//#include <util/delay.h>
#include "Pin.h"
#include "MTimer.h"
#include "UartBuf.h"
#include "Midi.h"
#include "A50akord.h"

FUSES = 
{
    .low = (FUSE_SUT1),
    .high = (FUSE_SPIEN),
    .extended = (FUSE_BODLEVEL1),
};
// DF DF FD


void Test(void)
{
	SetLED;										// LED on
	Delay_ms(400);								// delay 400 ms
	ClrLED;				 						// LED off
	Delay_ms(400);								// delay 400 ms
	UARTwrite(UARTpss,0x01);					// UART read/write
	UARTwrite(UARTpss,0x07);
	UARTwrite(UARTpss,0x1F);
	if (isUARTread(UARTpss)) {UARTwrite(UARTpss,UARTread(UARTpss));}
}


int main(void)
{
	PINopen();
	TIM1open();									// 1ms
	AKORDopen(UARTpss);
	
	sei();										// Enable Global Interrupt
	SetLED; Delay_ms(100); ClrLED; Delay_ms(100);
	SetLED; Delay_ms(100); ClrLED; Delay_ms(100);
	while(1)
	{
	  AKORDtask(UARTpss);
	}
}
