/*********************************************************************
 *
 *                  Midi
 *                  A50akord
 *
 * v.1.0 - 03.08.2021, Midi I/O, L.Pekny
 ********************************************************************/

#include <avr/io.h>
#include "MTimer.h"
#include "UartBuf.h"
#include "Midi.h"


tMIDImsg MIDImsg;
BYTE MIDIcmd;


BYTE MIDIsize(BYTE Cmd)								// Get data size of Msg
{
	BYTE Size;

	if (Cmd<0xF0) {Cmd&=0xF0;}						// Clear Channel
	switch(Cmd)
	{
	  case NoteOff: Size=2;
	  	break;
	  case NoteOn: Size=2;
	  	break;
	  case AfterTouchPoly: Size=2;
	  	break;
	  case ControlChange: Size=2;
	  	break;
	  case ProgramChange: Size=1;
	  	break;
	  case AfterTouchChan: Size=1;
	  	break;
	  case PitchBend: Size=2;
	  	break;
	  case SystemExclusive: Size=0x7F;
	  	break;
	  case TimeCodeQuarter: Size=1;
	  	break;
	  case SongPosition: Size=2;
	  	break;
	  case SongSelect: Size=1;
	  	break;
	  default: Size=0;
	  	break;
	}
	return Size;
}


tMIDImsg MIDIto14(WORD N16)
{
	tMIDImsg Nmsg;

	Nmsg.Dat1L=N16 & 0x7F;
	Nmsg.Dat2H=(N16<<1) & 0x7F;
	return Nmsg;
}


WORD MIDIto16(tMIDImsg Nmsg)
{
	WORD N16;

	N16=(Nmsg.Dat1L & 0x7F)+((Nmsg.Dat2H>>1) & 0x7F) ;
	return N16;
}


void MIDIopen(BYTE Ux)
{
	UARTspeed(Ux,3); UARTopen(Ux);					// Open Com, 31250 Bd
	MIDIcmd=0; MIDImsg.Cmd=0;
}


void MIDIclose(BYTE Ux)								// Close Com
{
	UARTclose(Ux);
}


tMIDImsg MIDIread(BYTE Ux)							// Read One Msg
{
	BYTE Size=0;
	tMIDImsg RDmsg;

	RDmsg.Cmd=InvalidMsg; RDmsg.Dat1L=0; RDmsg.Dat2H=0; RDmsg.Size=0;
	if (isUARTread(Ux))
	{
	  RDmsg.Cmd=UARTread(Ux);						// Read Cmd
	  if (RDmsg.Cmd<NoteOff)						// Param/Err?
	  {
	    if (MIDIcmd<NoteOff) {RDmsg.Cmd=InvalidMsg; return RDmsg;}	// Err

		RDmsg.Dat1L=RDmsg.Cmd;						// Dat1L,
		RDmsg.Cmd=MIDIcmd;							// Respawn Cmd

	    Size=MIDIsize(RDmsg.Cmd); RDmsg.Size=Size;

		if (!Size) {RDmsg.Cmd=InvalidMsg; return RDmsg;}

	    if (Size==2)								// Read 2.Data
	    {
	      if (isUARTreadT(Ux,Tms)) {RDmsg.Dat2H=UARTread(Ux);}	// Wait for char
		  else {RDmsg.Cmd=InvalidMsg; return RDmsg;}			// Timeout
	    }		 
	  }
	  else											// New Cmd
	  {
	    if (RDmsg.Cmd==SystemExclusive)				// Read Info string, skip
 	    {
	      while (Size<127)
	      {
	        if (isUARTreadT(Ux,Tms)) {RDmsg.Dat1L=UARTread(Ux);}	// Wait for char
		    else {Size=127;}										// Timeout
 
		    if (RDmsg.Dat1L==SystemExclusiveEnd) {Size=127;}		// End string
		    Size++;
		  }
		  return RDmsg;
	    }

	    Size=MIDIsize(RDmsg.Cmd); RDmsg.Size=Size;	// New Cmd

	    if (Size)									// Read 1.Data
	    {
	      MIDIcmd=RDmsg.Cmd;						// Backup only Cmd with param
	      if (isUARTreadT(Ux,Tms)) {RDmsg.Dat1L=UARTread(Ux);}		// Wait for char
		  else {RDmsg.Cmd=InvalidMsg; return RDmsg;}				// Timeout
	    }

	    if (Size==2)								// Read 2.Data
	    {
	      if (isUARTreadT(Ux,Tms)) {RDmsg.Dat2H=UARTread(Ux);}		// Wait for char
		  else {RDmsg.Cmd=InvalidMsg; return RDmsg;}				// Timeout
	    }
	  }
	}
	return RDmsg;
}


void MIDIwrite(BYTE Ux, tMIDImsg WRmsg)
{
	BYTE Size;

	if (WRmsg.Cmd<NoteOff) {return;}
	UARTwrite(Ux,WRmsg.Cmd);
	Size=MIDIsize(WRmsg.Cmd);
	if (Size) {UARTwrite(Ux,WRmsg.Dat1L);}
	if (Size==2) {UARTwrite(Ux,WRmsg.Dat2H);}
}

