/*********************************************************************
 *
 *                  Midi.h
 *                  A50akord
 *
 * v.1.0 - 07.08.2021, L.Pekny
 ********************************************************************/

#ifndef __MIDI_H__
#define __MIDI_H__


#define Tms						50			// 5ms timeout
#define KeyC3					48
#define KeyC4					60
#define KeyC5					72

typedef struct
{
	BYTE		Cmd;
	BYTE		Dat1L;
	BYTE		Dat2H;
	BYTE		Size;
} tMIDImsg;

extern tMIDImsg MIDImsg;

tMIDImsg MIDIto14(WORD N16);				// Convert WORD to 14b
WORD MIDIto16(tMIDImsg Nmsg);				// Convert 14b to WORD
BYTE MIDIsize(BYTE Cmd);					// Get data size of Msg
void MIDIopen(BYTE Ux);						// Open MIDI Com
void MIDIclose(BYTE Ux);					// Close MIDI Com
tMIDImsg MIDIread(BYTE Ux);					// Read one Msg
void MIDIwrite(BYTE Ux, tMIDImsg WRmsg);	// Write one Msg


// Msg type
enum MidiType
{
    InvalidMsg           		= 0x00,    // Error
    NoteOff               		= 0x80,    // Note Off (Key, Velocity)
    NoteOn                		= 0x90,    // Note On (Key, Velocity)
    AfterTouchPoly	      		= 0xA0,    // Polyphonic Key Pressure (Key, Pressure)
    ControlChange         		= 0xB0,    // Controller / Channel Mode (Controller, Value)
    ProgramChange         		= 0xC0,    // Program (Patch)
    AfterTouchChan	     		= 0xD0,    // Channel Pressure (Pressure)
    PitchBend             		= 0xE0,    // Pitch Bend 0-0x2000-0x3FFF (PitchL, PitchH)

    SystemExclusive       		= 0xF0,    // System Exclusive Start (D,D,....0xF7)
    TimeCodeQuarter		  		= 0xF1,    // Time Code Quarter Frame (mmmvvvv) Msg,Value
    SongPosition          		= 0xF2,    // Song Position Pointer (L, H)
    SongSelect            		= 0xF3,    // Song Select (Value)
    Undefined_F4          		= 0xF4,
    Undefined_F5          		= 0xF5,
    TuneRequest           		= 0xF6,    // Tune Request
	SystemExclusiveEnd    		= 0xF7,    // System Exclusive End

    Clock                 		= 0xF8,    // Real Time Clock 24/quarter
    Undefined_F9          		= 0xF9,
    Tick                  		= Undefined_F9,
    Start                 		= 0xFA,    // Start
    Continue              		= 0xFB,    // Continue
    Stop                  		= 0xFC,    // Stop
    Undefined_FD          		= 0xFD,
    ActiveSensing         		= 0xFE,    // Active <300 ms
    SystemReset           		= 0xFF     // System Reset
};


enum MidiControlChangeNumber
{
    // High resolution Continuous Controllers MSB ------------------------------
    BankSelect                  = 0,
    ModulationWheel             = 1,
    BreathController            = 2,
    // CC3 undefined
    FootController              = 4,
    PortamentoTime              = 5,
    DataEntry                   = 6,
    ChannelVolume               = 7,
    Balance                     = 8,
    // CC9 undefined
    Pan                         = 10,
    ExpressionController        = 11,
    EffectControl1              = 12,
    EffectControl2              = 13,
    // CC14 undefined
    // CC15 undefined
    GeneralPurpose1             = 16,
    GeneralPurpose2             = 17,
    GeneralPurpose3             = 18,
    GeneralPurpose4             = 19,
    // Low resolution Continuous Controllers LSB
	//							= 0+32 - 19+32

    // Switches ----------------------------------------------------------------
    Sustain                     = 64,
    Portamento                  = 65,
    Sostenuto                   = 66,
    SoftPedal                   = 67,
    Legato                      = 68,
    Hold                        = 69,

    // Low resolution continuous controllers -----------------------------------
    SoundController1            = 70,   // Synth: Sound Variation   FX: Exciter On/Off
    SoundController2            = 71,   // Synth: Harmonic Content  FX: Compressor On/Off
    SoundController3            = 72,   // Synth: Release Time      FX: Distortion On/Off
    SoundController4            = 73,   // Synth: Attack Time       FX: EQ On/Off
    SoundController5            = 74,   // Synth: Brightness        FX: Expander On/Off
    SoundController6            = 75,   // Synth: Decay Time        FX: Reverb On/Off
    SoundController7            = 76,   // Synth: Vibrato Rate      FX: Delay On/Off
    SoundController8            = 77,   // Synth: Vibrato Depth     FX: Pitch Transpose On/Off
    SoundController9            = 78,   // Synth: Vibrato Delay     FX: Flange/Chorus On/Off
    SoundController10           = 79,   // Synth: Undefined         FX: Special Effects On/Off
    GeneralPurpose5             = 80,
    GeneralPurpose6             = 81,
    GeneralPurpose7             = 82,
    GeneralPurpose8             = 83,
    PortamentoControl           = 84,
    // CC85 to CC90 undefined
    Effects1                    = 91,   // Reverb send level
    Effects2                    = 92,   // Tremolo depth
    Effects3                    = 93,   // Chorus send level
    Effects4                    = 94,   // Celeste depth
    Effects5                    = 95,   // Phaser depth
    DataIncrement               = 96,   // Data Entry +1
    DataDecrement               = 97,   // Data Entry -1
    NRPNLSB                     = 98,   // Non-Registered Parameter (LSB)
    NRPNMSB                     = 99,   // Non-Registered Parameter (MSB)
    RPNLSB                      = 100,  // Registered Parameter (LSB)
    RPNMSB                      = 101,  // Registered Parameter (MSB)

    // Channel Mode messages ---------------------------------------------------
    AllSoundOff                 = 120,
    ResetAllControllers         = 121,
    LocalControl                = 122,
    AllNotesOff                 = 123,
    OmniModeOff                 = 124,
    OmniModeOn                  = 125,
    MonoModeOn                  = 126,
    PolyModeOn                  = 127
}; 


enum GMvoice
{
	Piano						= 1,
	DrawOrgan					= 17,
	RockOrgan					= 19,
	Accordion					= 22,
	Harmonica					= 23,
	String						= 49,
	Violin						= 41,
	Clarinet					= 72,
	Flute						= 74,
	TenorSax					= 67,
	SquarLead					= 81,
	GMnone						= 255
};

#endif		

